/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.json.actions;

import java.util.Objects;
import org.json.JSONException;
import org.json.JSONObject;
import com.openexchange.exception.OXException;
import com.openexchange.file.storage.Document;
import com.openexchange.file.storage.FileStorageExceptionCodes;
import com.openexchange.file.storage.FileStorageFileAccess;
import com.openexchange.file.storage.composition.FileID;
import com.openexchange.file.storage.composition.IDBasedFileAccess;
import com.openexchange.file.storage.composition.IDBasedFileAccessFactory;
import com.openexchange.guard.mail.service.impl.PGPDocumentSessionKeyExtractor;
import com.openexchange.session.Session;
import com.openexchange.tools.servlet.OXJSONExceptionCodes;

/**
 * {@link FileSessionKeyExtractionHandler} extracts encrypted PGP session keys from a PGP encrypted file object
 *
 * @author <a href="mailto:benjamin.gruedelbach@open-xchange.com">Benjamin Gruedelbach</a>
 * @since v7,10.0
 */
public class FileSessionKeyExtractionHandler implements SessionKeyExtractionHandler {

    private static final String JSON_MODULE       = "module";
    private static final String DRIVE_MODULE      = "drive";
    private static final String JSON_FIELD_FOLDER = "folder";
    private static final String JSON_FIELD_ID     = "id";

    final private IDBasedFileAccessFactory fileAccessFactory;

    /**
     * Initializes a new {@link FileSessionKeyExtractionHandler}.
     *
     * @param fileAccessFactory
     */
    public FileSessionKeyExtractionHandler(IDBasedFileAccessFactory fileAccessFactory) {
        this.fileAccessFactory = Objects.requireNonNull(fileAccessFactory, "fileAccessFactory must not be null");
    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.guard.json.actions.SessionKeyExtractionHandler#canHandle(org.json.JSONObject)
     */
    @Override
    public boolean canHandle(JSONObject json) throws JSONException {
        return json != null && json.has(JSON_MODULE) && json.getString(JSON_MODULE).equals(DRIVE_MODULE);
    }

    /*
     * (non-Javadoc)
     *
     * @see com.openexchange.guard.json.actions.SessionKeyExtractionHandler#handle(org.json.JSONObject, com.openexchange.session.Session)
     */
    @Override
    public String[] handle(JSONObject json, Session session) throws OXException, JSONException {
        json = Objects.requireNonNull(json, "json must not be null");
        if(canHandle(json)) {

            if (!json.has(JSON_FIELD_ID)) {
                throw OXJSONExceptionCodes.MISSING_FIELD.create(JSON_FIELD_ID);
            }
            final String id = json.getString(JSON_FIELD_ID);
            FileID fileID = new FileID(id);

            if(json.has(JSON_FIELD_FOLDER) && fileID.getFolderId() == null) {
                fileID.setFolderId(json.getString(JSON_FIELD_FOLDER));
            }

            final IDBasedFileAccess fileAccess = fileAccessFactory.createAccess(session);
            final Document document = fileAccess.getDocumentAndMetadata(id, FileStorageFileAccess.CURRENT_VERSION);
            if(document != null) {
                return new PGPDocumentSessionKeyExtractor().getEncryptedSession(document, session);
            }
            else {
                throw FileStorageExceptionCodes.OPERATION_NOT_SUPPORTED.create(fileID.getService());
            }
        }
        return null;
    }
}
